package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.spark.utils.BaseUtil.{json_array, list_json}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row
import org.apache.spark.sql.expressions.{MutableAggregationBuffer, UserDefinedAggregateFunction}
import org.apache.spark.sql.types._

/**
 * @Description:company_case聚合
 * @author π
 * @date 2020/10/26 15:15
 */

class CompanyCaseAggs(max: Int) extends UserDefinedAggregateFunction {

  val flags = Seq("0", "7")
  val split = "\u0001"

  override def inputSchema: StructType = StructType(Array[StructField](
    StructField("judge_date", DataTypes.StringType)
    , StructField("case_reason_level4", DataTypes.StringType)
    , StructField("judge_year", DataTypes.StringType)
    , StructField("is_success", DataTypes.StringType)
    , StructField("plaintiff_info", DataTypes.StringType)
    , StructField("defendant_info", DataTypes.StringType)
    , StructField("flag", DataTypes.StringType)
  ))

  override def bufferSchema: StructType = StructType(
    Array[StructField](
      StructField("t1", DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType))
    )
  )

  override def dataType: DataType = DataTypes.createMapType(DataTypes.StringType, DataTypes.StringType)

  override def deterministic: Boolean = true

  override def initialize(buffer: MutableAggregationBuffer): Unit = {
    buffer.update(0, Map[String, String]())
  }

  override def update(buffer: MutableAggregationBuffer, input: Row): Unit = {
    if (buffer.size >= max) {
      return
    }
    if (input.size != 7) return
    val judge_date = input.getAs[String](0)
    val case_reason_level4 = input.getAs[String](1)
    val judge_year = input.getAs[String](2)
    val is_success = input.getAs[String](3)
    val plaintiff_info = input.getAs[String](4)
    val defendant_info = input.getAs[String](5)
    val flag = input.getAs[String](6)

    if (!flags.contains(flag)) {
      return
    }
    val map0 = buffer.getMap[String, String](0).toMap
    var map_new0 = scala.collection.mutable.Map[String, String](map0.toSeq: _*)

    if (flag.equals("0")) {
      if (StringUtils.isNotBlank(judge_date)) {
        map_new0 ++= Map(s"$judge_date" -> s"$case_reason_level4$split$judge_year$split$is_success$split$plaintiff_info$split$defendant_info$split$flag")
        buffer.update(0, map_new0)
      }
    } else if (flag.equals("7")) {
      map_new0 ++= Map(s"" -> s"$case_reason_level4$split$judge_year$split$is_success$split$plaintiff_info$split$defendant_info$split$flag")
      buffer.update(0, map_new0)
    }

  }

  override def merge(buffer1: MutableAggregationBuffer, buffer2: Row): Unit = {
    if (buffer1.size >= max) {
      return
    }
    buffer1(0) = buffer1.getAs[Map[String, String]](0) ++ buffer2.getAs[Map[String, String]](0)
  }

  override def evaluate(buffer: Row): Any = {
    val m0: Map[String, String] = buffer.getAs[Map[String, String]](0)

    if (m0.nonEmpty) {
      val keys = m0.keySet.filter(x => StringUtils.isNotBlank(x))
      if (keys.nonEmpty) {
        val minKey = keys.toSeq.min
        val maxKey = keys.toSeq.max

        val arr_min = m0.getOrElse(minKey, "").split(s"$split", -1)
        val arr_max = m0.getOrElse(maxKey, "").split(s"$split", -1)

        var is_success = arr_max(2)
        var flag = "0"
        if (m0.keySet.contains("")) {
          is_success = "胜"
          flag = "7"
        }
        val re = m0.filter(x => x._1.length > 0).map(y => {
          val arr = y._2.split(s"$split", -1)
          json_array(arr(3)) ++ json_array(arr(4))
        }).reduce(_ ++ _)

        return Map("judge_date" -> minKey
          , "case_reason_level4" -> arr_min(0)
          , "judge_year" -> arr_min(1)
          , "is_success" -> is_success
          , "plaintiff_info" -> list_json(re)
          , "defendant_info" -> "[]"
          , "flag" -> flag
        )
      }
    }
    Map("judge_date" -> ""
      , "case_reason_level4" -> ""
      , "judge_year" -> ""
      , "is_success" -> ""
      , "plaintiff_info" -> "[]"
      , "defendant_info" -> "[]"
      , "flag" -> ""
    )
  }
}
